@@ -76,6 +76,8 @@ extern const char *http_fv_con_cls;
 extern char http_fv_alv[128];
 extern char http_fv_cte[128];
 extern char http_url[128];
+extern char http_sig[128];
+extern char http_path[128];
 
 /*
  * Server configuration values
@@ -126,6 +128,7 @@ struct header {
 	char	rq_script[1024];	/* Request Script String */
 	char	rq_index[1024];		/* Request Index Filename */
 	char	rq_docrootv[1024];	/* Request Virtual Docroot */
+	char	rq_fv_usr[1024];	/* Request Remote User */
 	char	rq_fv_cte[1024];	/* Request Value Content-Type: */
 	char	rq_fv_ims[1024];	/* Request Value If-Modified-Since: */
 	char	rq_fv_ref[1024];	/* Request Value Referer: */
