@@ -93,7 +93,7 @@ static const char *icf = "<img src=/icons/file.gif>";
 int
 http_verify(const char *header, const char *cip, const int sfd)
 {
-	int		r, gmt;
+	int		r;
 	char		*h, *b, line[1024];
 	time_t		tnow;
 	struct tm	*t;
@@ -113,11 +113,9 @@ http_verify(const char *header, const char *cip, const int sfd)
 	if (r == 1) {
 		/* access log 1 client ip, date, request string */
 		time(&tnow);
-		t = gmtime(&tnow);
-		gmt = t->tm_hour;
 		t = localtime(&tnow);
 		snprintf(log_1, sizeof(log_1), "%s - - %s \"%s\" ", cip,
-		    sys_date(t, gmt), line);
+		    sys_date(t), line);
 	} else {
 		h = http_head(http_s_501, line, cip);
 		b = http_body(http_s_501, "", h);
@@ -145,11 +143,11 @@ http_proc(const char *header, char *body, const int hr, const int blen,
 {
 	DIR		*odir;
 	int		cpid, file, fds1[2], fds2[2];
-	int		i, r, s, bs, rd, sp, st, len, size;
+	int		i, r, s, bs, rp, rt, sp, st, len, size;
 	us_int		x_nph, x_fork, x_cpage;
 	char		*h = NULL, *x = NULL;
 	char		ch[1], image[128], tmp[1024], full[1024], status[1024];
-	char		b[BS], cpage[1024];
+	char		b[BS], cpage[1024], *cgiarg[2], *cgienv[64];
 	struct tm	*t = NULL;
 	struct stat	sta;
 	struct dirent	*rdir;
@@ -315,9 +313,9 @@ http_proc(const char *header, char *body, const int hr, const int blen,
 		strlcpy(h, rh->rp_header, rh->rp_hsize + bs);
 		if (rh->rp_status == 206)
 			lseek(file, rh->rp_foffs, SEEK_SET);
-		rd = read(file, h + strlen(h), bs);
+		rp = read(file, h + strlen(h), bs);
 		if (c[sfd].x_ssl) {
-			if ((sp = SSL_write(c[sfd].ssl, h, rh->rp_hsize + rd))
+			if ((sp = SSL_write(c[sfd].ssl, h, rh->rp_hsize + rp))
 			    == -1) {
 				free(h);
 				free(rh);
@@ -325,7 +323,7 @@ http_proc(const char *header, char *body, const int hr, const int blen,
 				return -1;
 			}
 		} else {
-			if ((sp = sys_write(sfd, h, rh->rp_hsize + rd)) == -1) {
+			if ((sp = sys_write(sfd, h, rh->rp_hsize + rp)) == -1) {
 				free(h);
 				free(rh);
 				close(file);
@@ -423,7 +421,80 @@ http_proc(const char *header, char *body, const int hr, const int blen,
 			chdir(rh->rq_filep);
 			dup2(fds1[0], STDIN_FILENO);
 			dup2(fds2[1], STDOUT_FILENO);
-			execl(rh->rq_filef, rh->rq_files, (char *)NULL);
+
+			/* build cgi environment array */
+			i = 0;
+			cgiarg[0] = rh->rq_files;
+			cgiarg[1] = '\0';
+			cgienv[i++] = sys_benv("DOCUMENT_ROOT=%s",
+			    rh->rq_docrootv);
+			cgienv[i++] = sys_benv("GATEWAY_INTERFACE=%s",
+			    http_fv_cgi);
+			cgienv[i++] = sys_benv("PATH=%s",
+			    http_path);
+			cgienv[i++] = sys_benv("REMOTE_ADDR=%s",
+			    c[sfd].ip);
+			cgienv[i++] = sys_benv("REMOTE_PORT=%s",
+			    c[sfd].pt);
+			cgienv[i++] = sys_benv("REQUEST_METHOD=%s",
+			    rh->rq_method);
+			cgienv[i++] = sys_benv("REQUEST_URI=%s",
+			    rh->rq_uri);
+			cgienv[i++] = sys_benv("SCRIPT_FILENAME=%s",
+			    rh->rq_filef);
+			cgienv[i++] = sys_benv("SCRIPT_NAME=%s",
+			    rh->rq_script);
+			cgienv[i++] = sys_benv("SERVER_ADMIN=%s",
+			    config.serveradmin);
+			cgienv[i++] = sys_benv("SERVER_NAME=%s",
+			    config.servername);
+			cgienv[i++] = sys_benv("SERVER_PROTOCOL=%s",
+			    http_fv_pro);
+			cgienv[i++] = sys_benv("SERVER_SIGNATURE=%s",
+			    http_sig);
+			cgienv[i++] = sys_benv("SERVER_SOFTWARE=%s",
+			    http_fv_srv);
+			if (rh->x_clt)
+				cgienv[i++] = sys_benv("CONTENT_LENGTH=%s",
+				    rh->rq_fv_clt);
+			if (rh->x_con)
+				cgienv[i++] = sys_benv("HTTP_CONNECTION=%s",
+				    rh->rq_fv_con);
+			if (rh->x_cok)
+				cgienv[i++] = sys_benv("HTTP_COOKIE=%s",
+				    rh->rq_fv_cok);
+			if (rh->x_hos)
+				cgienv[i++] = sys_benv("HTTP_HOST=%s",
+				    rh->rq_fv_hos);
+			if (rh->x_agt)
+				cgienv[i++] = sys_benv("HTTP_USER_AGENT=%s",
+				    rh->rq_fv_agt);
+			if (c[sfd].x_ssl)
+				cgienv[i++] = sys_benv("HTTPS=on");
+			if (rh->x_opt)
+				cgienv[i++] = sys_benv("PATH_INFO=%s",
+				    rh->rq_option);
+			if (rh->x_qry)
+				cgienv[i++] = sys_benv("QUERY_STRING=%s",
+				    rh->rq_query);
+			if (rh->x_aut)
+				cgienv[i++] = sys_benv("REMOTE_USER=%s",
+				    rh->rq_fv_usr);
+			if (c[sfd].x_ip6)
+				cgienv[i++] = sys_benv("SERVER_ADDR=%s",
+				    config.serverip6);
+			else
+				cgienv[i++] = sys_benv("SERVER_ADDR=%s",
+				    config.serverip4);
+			if (c[sfd].x_ssl)
+				cgienv[i++] = sys_benv("SERVER_PORT=%s",
+				    config.sslportc);
+			else
+				cgienv[i++] = sys_benv("SERVER_PORT=%s",
+				    config.serverportc);
+			cgienv[i++] = '\0';
+
+			execve(rh->rq_filef, cgiarg, cgienv);
 			exit(0);
 		}
 
@@ -432,8 +503,42 @@ http_proc(const char *header, char *body, const int hr, const int blen,
 		close(fds2[1]);
 
 		/* if post send data to cgis stdin */
-		if (strcmp(rh->rq_method, "POST") == 0)
-			sys_write(fds1[1], body, blen);
+		if (strcmp(rh->rq_method, "POST") == 0) {
+			rp = 0;
+			rt = 0;
+			size = atoi(rh->rq_fv_clt);
+
+			if (blen > 0) {
+				r = http_body_comp(body, blen, blen, size);
+				if (r > 0)
+					sys_write(fds1[1], body, r);
+				else
+					sys_write(fds1[1], body, blen);
+			}
+
+			if (blen == 0 || r == 0) {
+				rt += blen;
+				while (1) {
+					if (c[sfd].x_ssl) {
+						rp = SSL_read(c[sfd].ssl, b,
+						    sizeof(b));	
+					} else {
+						rp = sys_read(sfd, b,
+						    sizeof(b));
+					}
+					if (rp < 1)
+						break;
+					rt += rp;
+					r = http_body_comp(b, rp, rt, size);
+					if (r > 0) {
+						sys_write(fds1[1], b, r);
+						break;
+					} else {
+						sys_write(fds1[1], b, rp);
+					}
+				}
+			}
+		}
 		/* close fd to cgi stdin */
 		close(fds1[1]);
 
@@ -442,7 +547,7 @@ http_proc(const char *header, char *body, const int hr, const int blen,
 
 		/* parse cgi header */
 		for (i = 0; i < sizeof(b); i++) {
-			if ((rd = read(fds2[0], ch, 1)) < 1) {
+			if ((rp = read(fds2[0], ch, 1)) < 1) {
 				flogd(config.logserver, "[error] %s "
 				    "sent a bad cgi header\n", rh->rq_uri);
 				exit(1);
@@ -501,7 +606,7 @@ http_proc(const char *header, char *body, const int hr, const int blen,
 
 			/* data */
 			memset(b, 0, sizeof(b));
-			if ((rd = read(fds2[0], b, sizeof(b))) < 1)
+			if ((rp = read(fds2[0], b, sizeof(b))) < 1)
 				break;
 			if (x_nph)
 				x = b;
@@ -650,7 +755,7 @@ http_proc(const char *header, char *body, const int hr, const int blen,
 
 			/* create html footer */
 			snprintf(b, bs + 1, "</table>\n<hr>\n%s\n</body>\n"
-			    "</html>", getenv("SERVER_SIGNATURE"));
+			    "</html>", http_sig);
 			x = http_chunk(b);
 			if (c[sfd].x_ssl) {
 				if ((sp = SSL_write(c[sfd].ssl, x, strlen(x)))
@@ -843,25 +948,14 @@ http_header_comp(char *header)
  *	0 = body not complete, <bytes of body> = body complete
  */
 int
-http_body_comp(const char *header, char *body, const int blen)
+http_body_comp(char *body, const int blen, const int brec, const int hlen)
 {
-	int	i, r, hlen;
-	char	line[1024], option[1024], clength[16];
+	int	r;
 	
 	r = 0;
 
-	/* get content length from header */
-	for (i = 2; strcutl(line, header, i, sizeof(line)) != -1; i++) {
-		strcutw(option, line, 1, sizeof(option));
-		if (strcasecmp(option, http_fn_clt) == 0) {
-			strcuts(clength, line, ' ', '\0', sizeof(clength));
-			break;
-		}
-	}
-	hlen = atoi(clength);
-
 	/* handle the post body termination mess */
-	if (blen >= hlen) {
+	if (brec >= hlen) {
 		if (body[blen - 2] == '\r')
 			r = blen - 2;
 		else if (body[blen - 1] == '\n')
@@ -922,17 +1016,14 @@ http_access_htaccess(char *dst, const char *rootdir, const int dsize)
 int
 http_alog(const int sfd, const int hr)
 {
-	int		gmt;
 	time_t		tnow;
 	struct tm	*t;
 
 	time(&tnow);
-	t = gmtime(&tnow);
-	gmt = t->tm_hour;
 	t = localtime(&tnow);
 
 	snprintf(log_1, sizeof(log_1), "%s - - %s \"%s\" %d %d",
-	    c[sfd].ip, sys_date(t, gmt), c[sfd].plreq[hr], c[sfd].status[hr],
+	    c[sfd].ip, sys_date(t), c[sfd].plreq[hr], c[sfd].status[hr],
 	    c[sfd].pfds[hr]);
 	snprintf(log_2, sizeof(log_2), "\"%s\" \"%s\"",
 	    c[sfd].plref[hr], c[sfd].plagt[hr]);
@@ -978,20 +1069,12 @@ http_chunk(const char *string)
 char *
 http_date(struct tm *t)
 {
-	int		hour, min, sec, year, mon, mday, wday;
 	char		*d;
 	static char	date[64];
 
-	hour = t->tm_hour;
-	min = t->tm_min;
-	sec = t->tm_sec;
-	year = t->tm_year;
-	mon = t->tm_mon;
-	mday = t->tm_mday;
-	wday = t->tm_wday;
-
-	snprintf(date, 64, "%s, %02d %s %d %02d:%02d:%02d %s",
-	    day[wday], mday, month[mon], year+1900, hour, min, sec, t->tm_zone);
+	snprintf(date, sizeof(date), "%s, %02d %s %d %02d:%02d:%02d %s",
+	    day[t->tm_wday], t->tm_mday, month[t->tm_mon], t->tm_year + 1900,
+	    t->tm_hour, t->tm_min, t->tm_sec, t->tm_zone);
 
 	d = date;
 
@@ -1053,7 +1136,7 @@ http_uridecode(const char *uri)
 char *
 http_head(const char *status, const char *request, const char *cip)
 {
-	int		gmt, status_nr;
+	int		status_nr;
 	char		*h, header[8192], tmp[1024], date[128];
 	time_t		tnow;
 	struct tm	*t;
@@ -1097,13 +1180,11 @@ http_head(const char *status, const char *request, const char *cip)
 
 	/* get date and time */
 	time(&tnow);
-	t = gmtime(&tnow);
-	gmt = t->tm_hour;
 	t = localtime(&tnow);
 
 	/* access log 1 */
 	snprintf(log_1, sizeof(log_1), "%s - - %s \"%s\" %d -", cip,
-	    sys_date(t, gmt), request, status_nr);
+	    sys_date(t), request, status_nr);
 
 	/* access log 2 */
 	strlcpy(log_2, "\"-\" \"-\"", sizeof(log_2));
@@ -1129,7 +1210,7 @@ http_body(const char *title, const char *msg, const char *header)
 	/* html */
 	snprintf(body, sizeof(body), "<html>\n<head>\n<title>%s</title>\n"
 	    "</head>\n<body>\n<h1>%s</h1>\n%s\n<hr>\n%s\n</body>\n</html>",
-	    title, title, msg, getenv("SERVER_SIGNATURE"));
+	    title, title, msg, http_sig);
 
 	/* chunk html */
 	body_chunk = http_chunk(body);
@@ -1166,7 +1247,7 @@ http_header(const char *header_data, const char *force_status,
 	char		alias[1024], cgi_path[1024], cgi_full[1024];
 	char		docroot[1024], file_path[1024];
 	char		acc_base64[1024], acc_file[1024];
-	char		acc_userpw[1024], acc_user[1024], acc_pw[1024];
+	char		acc_userpw[1024], acc_pw[1024];
 	char		tmp[1024], status[128];
 	char		*x, *type = NULL;
 	time_t		tnow;
@@ -1450,11 +1531,11 @@ http_header(const char *header_data, const char *force_status,
 			    sizeof(acc_base64));
 			strb64d(acc_userpw, acc_base64, sizeof(acc_userpw));
 			/* separate user and password */
-			strcuts(acc_user, acc_userpw, '\0', ':',
-			    sizeof(acc_user));
+			strcuts(h->rq_fv_usr, acc_userpw, '\0', ':',
+			    sizeof(h->rq_fv_usr));
 			strcuts(acc_pw, acc_userpw, ':', '\0', sizeof(acc_pw));
 			/* authenticate */
-			if ((r = sys_access_auth(acc_user, acc_pw)) != 1)
+			if ((r = sys_access_auth(h->rq_fv_usr, acc_pw)) != 1)
 				/* failed */
 				h->x_aut = 0;
 		}
@@ -1564,76 +1645,13 @@ http_header(const char *header_data, const char *force_status,
 	/* return the size of our header */
 	h->rp_hsize = strlen(h->rp_header);
 
-	/* set dynamic cgi environment vars */
-	if (h->x_cgi) {
-		/* cleanup first */
-		unsetenv("CONTENT_LENGTH");
-		unsetenv("DOCUMENT_ROOT");
-		unsetenv("HTTP_COOKIE");
-		unsetenv("HTTP_USER_AGENT");
-		unsetenv("HTTP_CONNECTION");
-		unsetenv("HTTP_HOST");
-		unsetenv("HTTPS");
-		unsetenv("PATH_INFO");
-		unsetenv("QUERY_STRING");
-		unsetenv("REMOTE_ADDR");
-		unsetenv("REMOTE_PORT");
-		unsetenv("REMOTE_USER");
-		unsetenv("REQUEST_METHOD");
-		unsetenv("REQUEST_URI");
-		unsetenv("SCRIPT_FILENAME");
-		unsetenv("SCRIPT_NAME");
-		unsetenv("SERVER_ADDR");
-		unsetenv("SERVER_PORT");
-
-		/* mandatory */
-		setenv("DOCUMENT_ROOT", docroot, 1);
-		setenv("REMOTE_ADDR", c[sfd].ip, 1);
-		setenv("REMOTE_PORT", c[sfd].pt, 1);
-		setenv("REQUEST_METHOD", h->rq_method, 1);
-		setenv("REQUEST_URI", h->rq_uri, 1);
-		setenv("SCRIPT_FILENAME", h->rq_filef, 1);
-		setenv("SCRIPT_NAME", h->rq_script, 1);
-		if (c[sfd].x_ip6)
-			setenv("SERVER_ADDR", config.serverip6, 1);
-		else
-			setenv("SERVER_ADDR", config.serverip4, 1);
-		if (c[sfd].x_ssl)
-			setenv("SERVER_PORT", config.sslportc, 1);
-		else
-			setenv("SERVER_PORT", config.serverportc, 1);
-
-		/* optional */
-		if (h->x_clt) {
-			snprintf(tmp, sizeof(tmp), "%d", blen);
-			setenv("CONTENT_LENGTH", tmp, 1);
-		}
-		if (h->x_cok)
-			setenv("HTTP_COOKIE", h->rq_fv_cok, 1);
-		if (h->x_con)
-			setenv("HTTP_CONNECTION", h->rq_fv_con, 1);
-		if (h->x_hos)
-			setenv("HTTP_HOST", h->rq_fv_hos, 1);
-		if (h->x_agt)
-			setenv("HTTP_USER_AGENT", h->rq_fv_agt, 1);
-		if (c[sfd].x_ssl)
-			setenv("HTTPS", "on", 1);
-		if (h->x_opt)
-			setenv("PATH_INFO", h->rq_option, 1);
-		if (h->x_qry)
-			setenv("QUERY_STRING", h->rq_query, 1);
-		if (h->x_aut)
-			setenv("REMOTE_USER", acc_user, 1);
-	}
-
 	/* set signature variable */
 	if (h->x_cgi || h->x_idx || (h->rp_status != 200 &&
 	    h->rp_status != 206 && h->rp_status != 314)) {
-		snprintf(tmp, sizeof(tmp),
+		snprintf(http_sig, sizeof(http_sig),
 		    "<address>%s at %s Port %s</address>", http_fv_srv,
 		    h->rq_fv_hos, c[sfd].x_ssl ? config.sslportc :
 		    config.serverportc);
-		setenv("SERVER_SIGNATURE", tmp, 1);
 	}
 		    
 	return h;
