@@ -23,11 +23,11 @@
 
 #include <netinet/in.h>
 #include <arpa/inet.h>
-#include <netdb.h>
 
 #include <err.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <netdb.h>
 #include <openssl/ssl.h>
 #include <pwd.h>
 #include <signal.h>
@@ -98,7 +98,7 @@ const char *http_fn_aut		= "Authorization:";
 const char *http_fn_auw		= "WWW-Authenticate:";
 const char *http_fn_ran		= "Range:";
 const char *http_fn_rac		= "Content-Range:";
-const char *http_fv_srv		= "Nostromo/1.7.4 (Unix)";
+const char *http_fv_srv		= "Nostromo/1.7.5 (Unix)";
 const char *http_fv_pro		= "HTTP/1.1";
 const char *http_fv_cgi		= "CGI/1.1";
 const char *http_fv_teg		= "chunked";
@@ -290,7 +290,7 @@ main(int argc, char *argv[])
 	int			sdlis1, sdlis2, sdlis3, sdlis4;
 	int			sdmax, sdnew, sdnow;
 	int			i, j, k, r, s, ch, rd, rp, rt, wr, post, size;
-	int			fd, erro, mode, opt_c, opt_r, opt_4, opt_6;
+	int			fd, erro, opt_c, opt_r, opt_4, opt_6;
 	char			in[HDN * HDS + 1], tmp[HDS + 1];
 	char			header[HDN][HDS + 1];
 	char			*b, *h, *body = NULL;
@@ -578,11 +578,10 @@ main(int argc, char *argv[])
 		if ((sdlis1 = socket(AF_INET, SOCK_STREAM, 0)) == -1)
 			err(1, "socket");
 
-		/* increase socket send buffer */
-		size = HDN * BS;
-		if (setsockopt(sdlis1, SOL_SOCKET, SO_SNDBUF, &size,
-		    sizeof(size)) == -1)
-			err(1, "setsockopt");
+		/* set socket non-blocking */
+		if (fcntl(sdlis1, F_SETFL, fcntl(sdlis1, F_GETFL) |
+		    O_NONBLOCK) == -1)
+			err(1, "fcntl");
 		/* allow rapid reuse of the address */
 		size = 1;
 		if (setsockopt(sdlis1, SOL_SOCKET, SO_REUSEADDR, (char *) &size,
@@ -616,11 +615,10 @@ main(int argc, char *argv[])
 		if ((sdlis2 = socket(AF_INET, SOCK_STREAM, 0)) == -1)
 			err(1, "socket");
 
-		/* increase socket send buffer */
-		size = HDN * BS;
-		if (setsockopt(sdlis2, SOL_SOCKET, SO_SNDBUF, &size,
-		    sizeof(size)) == -1)
-			err(1, "setsockopt");
+		/* set socket non-blocking */
+		if (fcntl(sdlis2, F_SETFL, fcntl(sdlis2, F_GETFL) |
+		    O_NONBLOCK) == -1)
+			err(1, "fcntl");
 		/* allow rapid reuse of the address */
 		size = 1;
 		if (setsockopt(sdlis2, SOL_SOCKET, SO_REUSEADDR, (char *) &size,
@@ -654,11 +652,10 @@ main(int argc, char *argv[])
 		if ((sdlis3 = socket(AF_INET6, SOCK_STREAM, 0)) == -1)
 			err(1, "socket");
 
-		/* increase socket send buffer */
-		size = HDN * BS;
-		if (setsockopt(sdlis3, SOL_SOCKET, SO_SNDBUF, &size,
-		    sizeof(size)) == -1)
-			err(1, "setsockopt");
+		/* set socket non-blocking */
+		if (fcntl(sdlis3, F_SETFL, fcntl(sdlis3, F_GETFL) |
+		    O_NONBLOCK) == -1)
+			err(1, "fcntl");
 		/* allow rapid reuse of the address */
 		size = 1;
 		if (setsockopt(sdlis3, SOL_SOCKET, SO_REUSEADDR, (char *) &size,
@@ -697,11 +694,10 @@ main(int argc, char *argv[])
 		if ((sdlis4 = socket(AF_INET6, SOCK_STREAM, 0)) == -1)
 			err(1, "socket");
 
-		/* increase socket send buffer */
-		size = HDN * BS;
-		if (setsockopt(sdlis4, SOL_SOCKET, SO_SNDBUF, &size,
-		    sizeof(size)) == -1)
-			err(1, "setsockopt");
+		/* set socket non-blocking */
+		if (fcntl(sdlis4, F_SETFL, fcntl(sdlis4, F_GETFL) |
+		    O_NONBLOCK) == -1)
+			err(1, "fcntl");
 		/* allow rapid reuse of the address */
 		size = 1;
 		if (setsockopt(sdlis4, SOL_SOCKET, SO_REUSEADDR, (char *) &size,
@@ -881,6 +877,7 @@ main(int argc, char *argv[])
 			 */
 			if (rd > 0 && (sdnow == sdlis1 || sdnow == sdlis2 ||
 			    sdnow == sdlis3 || sdnow == sdlis4)) {
+				/* accept */
 				if (sdnow == sdlis3 || sdnow == sdlis4) {
 					ssize = sizeof(ca6);
 					if ((sdnew = accept(sdnow,
@@ -895,6 +892,15 @@ main(int argc, char *argv[])
 						continue;
 				}
 
+				/* set socket options */
+				if (fcntl(sdnew, F_SETFL, fcntl(sdnew,
+				    F_GETFL) | O_NONBLOCK) == -1)
+					continue;
+				size = HDN * BS;
+				if (setsockopt(sdnew, SOL_SOCKET, SO_SNDBUF,
+				    &size, sizeof(size)) == -1)
+					continue;
+
 				/* SSL ready */
 				if (sdnow == sdlis2 || sdnow == sdlis4) {
 					c[sdnew].ssl = SSL_new(ssl_ctx);
@@ -955,10 +961,7 @@ main(int argc, char *argv[])
 			 * SSL handshake
 			 */
 			if (c[sdnow].x_ssl == 1) {
-				mode = fcntl(sdnow, F_GETFL);
-				fcntl(sdnow, F_SETFL, O_NONBLOCK);
 				r = SSL_accept(c[sdnow].ssl);
-				fcntl(sdnow, F_SETFL, mode);
 				if (r == 1) {
 					c[sdnow].x_ssl = 2;
 					continue;
