@@ -147,21 +147,35 @@ sys_access_auth(const char *clientuser, const char *clientpw)
 
 /*
  * sys_read()
- *	read data form network to buffer
+ *	synchronous read
  * Return:
- *	<bytes> = success, 0 = connection closed, -1 = error
+ *	<bytes> = success, 0 = end-of-file, -1 = error
  */
 int
 sys_read(const int sfd, char *buf, const int len)
 {
-	int	r;
+	int		r;
+	fd_set		seto_r;
+	struct timeval	tv;
 
+	FD_ZERO(&seto_r);
+	FD_SET(sfd, &seto_r);
 	while (1) {
+		tv.tv_sec = 1;
+		tv.tv_usec = 0;
+		r = select(sfd + 1, &seto_r, NULL, NULL, &tv);
+		if (r == -1) {
+			if (errno == EINTR)
+				continue;
+			return -1;
+		}
+		if (r == 0)
+			return -1;
 		r = read(sfd, buf, len);
 		if (r == -1) {
 			if (errno == EINTR)
 				continue;
-			break;
+			return -1;
 		}
 		if (r == 0)
 			break;
@@ -174,24 +188,37 @@ sys_read(const int sfd, char *buf, const int len)
 
 /*
  * sys_write()
- *	send all data from buffer to network
+ *	synchronous write
  * Return:
  *	<bytes> = success, -1 = error
  */
 int
 sys_write(const int sfd, const char *buf, const int len)
 {
-	int	r, sent;
+	int		r, sent;
+	fd_set		seto_w;
+	struct timeval	tv;
 
 	sent = 0;
 
+	FD_ZERO(&seto_w);
+	FD_SET(sfd, &seto_w);
 	while (1) {
+		tv.tv_sec = 1;
+		tv.tv_usec = 0;
+		r = select(sfd + 1, NULL, &seto_w, NULL, &tv);
+		if (r == -1) {
+			if (errno == EINTR)
+				continue;
+			return -1;
+		}
+		if (r == 0)
+			return -1;
 		r = write(sfd, buf + sent, len - sent);
 		if (r == -1) {
 			if (errno == EINTR)
 				continue;
-			sent = r;
-			break;
+			return -1;
 		}
 		if (r > 0)
 			sent += r;
